<?php
/**
 * @author Erlend Strømsvik - Ny Media AS
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package moodle multiauth
 *
 * Authentication Plugin: simplesaml Authentication
 *
 * Authentication using SAML2 with SimpleSAMLphp.
 *
 * Based on plugins made by Sergio Gómez (moodle_ssp) and Martin Dougiamas (Shibboleth).
 *
 * 2008-10  Created
 *
 * Modifyed for use with AAI@EduHr
 * by mijo@srce.hr
 * 2008-11-25
**/

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.');    ///  It must be included from a Moodle page
}

require_once($CFG->libdir.'/authlib.php');

/**
 * SimpleSAML authentication plugin.
**/
class auth_plugin_simplesaml extends auth_plugin_base {

    /**
    * Constructor.
    */
    function auth_plugin_simplesaml() {
        $this->authtype = 'simplesaml';
        $this->config = get_config('auth/simplesaml');
    }

    /**
    * Returns true if the username and password work and false if they are
    * wrong or don't exist.
    *
    * @param string $username The username (with system magic quotes)
    * @param string $password The password (with system magic quotes)
    * @return bool Authentication success or failure.
    */
    function user_login($username, $password) {
        // if true, user_login was initiated by simplesaml/index.php
        if($GLOBALS['simplesaml_login']) {
            unset($GLOBALS['simplesaml_login']);
            return TRUE;
        }
        return FALSE;
    }


    /**
    * Returns the user information for 'external' users. In this case the
    * attributes provided by simplesaml
    *
    * @return array $result Associative array of user data
    */
     function get_userinfo($username){
     	if($GLOBALS['simplesaml_login_attributes']) {
	    	$attributes=$this->map_simplesaml_attributes();
	    	$user_info_arr=$GLOBALS['simplesaml_login_attributes'];
	    	foreach ($attributes as $moodle_name=>$simplesaml_name){
	    		if(is_array($user_info_arr[$simplesaml_name])){
	    			$moodle_info[$moodle_name]=join(', ', $user_info_arr[$simplesaml_name]);
	    		} else {
	    			$moodle_info[$moodle_name]=$user_info_arr[$simplesaml_name];
	    		}
	    	}
	    	$moodle_info['country']='HR';
	    	unset($GLOBALS['simplesaml_login_attributes']);

	    	return $moodle_info;
     	}
     	return FALSE;
    }

    /*
    * Returns array containg attribute mappings between Moodle and simplesaml.
    */

    function map_simplesaml_attributes () {
        $fields = array("firstname", "lastname", "email", "city", "institution", "phone1", "phone2", "department", "address");
        $customfields = $this->get_custom_user_profile_fields();

        if (!empty($customfields) && !empty($fields)) {
        	$fields = array_merge($fields, $customfields);
        }

        $moodleattributes = array();

        foreach ($fields as $field) {
        	if (!empty($this->config->{"field_map_$field"})) {
        		$moodleattributes[$field] = trim($this->config->{"field_map_$field"});
        		if (preg_match('/,/', $moodleattributes[$field])) {
        			$moodleattributes[$field] = explode(',', $moodleattributes[$field]); // split ?
        		}
        	}
        }

        $moodleattributes['username'] = 'hrEduPersonUniqueID';

        return $moodleattributes;
    }


    /**
    * Returns true if this authentication plugin is 'internal'.
    *
    * @return bool
    */
    function is_internal() {
        return false;
    }

    /**
    * Returns true if this authentication plugin can change the user's
    * password.
    *
    * @return bool
    */
    function can_change_password() {
        return false;
    }

    function loginpage_hook() {
        // Prevent username from being shown on login page after logout
        $GLOBALS['CFG']->nolastloggedin = true;

        return;
    }

    function logoutpage_hook() {
		if($this->config->dologout) {
            set_moodle_cookie('nobody');
            require_logout();
            redirect($GLOBALS['CFG']->wwwroot.'/auth/simplesaml/index.php?logout=1');
        }
    }

    /**
    * Prints a form for configuring this authentication plugin.
    *
    * This function is called from admin/auth.php, and outputs a full page with
    * a form for configuring this plugin.
    *
    * @param array $page An object containing all the data for this page.
    */

    function config_form($config, $err, $user_fields) {
        include "config.html";
    }


    /**
    * Processes and stores configuration data for this authentication plugin.
    *
    *
    * @param object $config Configuration object
    */
    function process_config($config) {
        // set to defaults if undefined
        if (!isset ($config->dologout)) {
            $config->dologout = '';
        }
        if (!isset ($config->notshowusername)) {
            $config->notshowusername = '';
        }

        // save settings
        set_config('dologout',        $config->dologout,        'auth/simplesaml');
        set_config('notshowusername', $config->notshowusername, 'auth/simplesaml');

        return true;
    }

    /**
    * Cleans and returns first of potential many values (multi-valued attributes)
    *
    * @param string $string Possibly multi-valued attribute from simplesaml
    */
    function get_first_string($string) {
        $list = split( ';', $string);
        $clean_string = trim($list[0]);

        return $clean_string;
    }
}

?>
